@@ -24,7 +24,7 @@ except ImportError as e:
     from filter import cigar
     from filter import qualities_cigar
 
-__version__ = '0.4.8'
+__version__ = '0.4.12'
 
 def downstream_phase_detection(variants, segment, record, quality):
     """
@@ -285,7 +285,7 @@ class Pair():
             self.rec_1 = pysam.AlignedSegment.fromstring(self.rec_1, header)
             self.rec_2 = pysam.AlignedSegment.fromstring(self.rec_2, header)
 
-    def classify(self, masking=0, quality=0, vcf=None):
+    def classify(self, masking=0, quality=0, vcf=None, mvr_vcf_filepath=None):
         """
         Determine the type of recombination event that occursed in this read pair.
 
@@ -318,6 +318,9 @@ class Pair():
         vcf: cyvcf2.VCF, optional
             pre-initialized ``cyvcf2.VCF`` for this ``Pair`` to use
             when classifying, highly recommended
+        mvr_vcf_filepath : str, optional
+            path to VCF containing sites that are invariant between parents for
+            more precise mismatch variant ratio calculations
         """
         # create new VCF object if none is provided
         if not vcf:
@@ -384,7 +387,7 @@ class Pair():
                 tupl[0] = samples[1]
 
         # set descriptive + quality attributes post-classification
-        self._describe(haplotypes, vcf)
+        self._describe(haplotypes, vcf, mvr_vcf_filepath)
 
         # if gene conversion - set descriptive info
         if 'gene_conversion' in [self.call, self.masked_call]:
@@ -481,7 +484,7 @@ class Pair():
         
         return condensed
 
-    def _describe(self, haplotypes, vcf):
+    def _describe(self, haplotypes, vcf, mvr_vcf_filepath):
         """
         Internal method to calculate various attributes of ``Pair``. Sets
         the following attributes:
@@ -512,12 +515,14 @@ class Pair():
             haplotype listing generated from self.masked_condensed
         vcf : cyvcf2.VCF
             VCF file object generated by ``classify()``
+        mvr_vcf_filepath : str
+            optional path to VCF not yet filtered by vcfprep for MVR calc
         """
         if not self.call:
             self.classify()
 
         # use helper function to calculate unexpected mismatch count
-        self.mismatch_variant_ratio = self._get_mismatch_variant_ratio()
+        self.mismatch_variant_ratio = self._get_mismatch_variant_ratio(mvr_vcf_filepath)
 
         # calculate average number of variants per haplotype and create dict of counts
         self.variants_per_haplotype = len(self.variants_filt) / max(len(haplotypes), 1)
@@ -564,31 +569,32 @@ class Pair():
         read_bounds = [
             self.rec_1.reference_start, self.rec_1.reference_start + len(self.segment_1),
             self.rec_2.reference_start, self.rec_2.reference_start + len(self.segment_2)]
-        if not self.overlap:
-            self.min_end_proximity = min(
-                [abs(pos - bound) for bound in read_bounds for pos in self.phase_change_variants])
-        elif self.overlap: 
-            # only applies to concordant phase change variant overlap
-            # discordant overlapping sites are removed anyways
-            # very brute force... sigh
-            current_min = None
-            for pos in self.phase_change_variants:
-                # iterate through variants and update min variable
-                if pos in self.overlapping_sites:
-                    # find longer read end
-                    read1_min = min([abs(pos - bound) for bound in read_bounds[:2]])
-                    read2_min = min([abs(pos - bound) for bound in read_bounds[2:]])
-                    pos_min = max(read1_min, read2_min)
-                elif pos not in self.overlapping_sites:
-                    pos_min = min([abs(pos - bound) for bound in read_bounds])
-                # update current min
-                if not current_min:
-                    current_min = pos_min
-                elif current_min > pos_min: # new min found
-                    current_min = pos_min
-                elif current_min <= pos_min:
-                    continue
-            self.min_end_proximity = current_min
+        # very brute force but handles overlap regions correctly
+        current_min = None
+        for pos in self.phase_change_variants:
+            # iterate through variants and update min variable
+            if pos in self.overlapping_sites:
+                # find longer read end
+                read1_min = min([abs(pos - bound) for bound in read_bounds[:2]])
+                read2_min = min([abs(pos - bound) for bound in read_bounds[2:]])
+                pos_min = max(read1_min, read2_min)
+            elif pos not in self.overlapping_sites:
+                # site only present on one read - need to just use those boundaries
+                if read_bounds[0] <= pos <= read_bounds[1]:
+                    pos_min = min([abs(pos - bound) for bound in read_bounds[:2]])
+                elif read_bounds[2] <= pos <= read_bounds[3]:
+                    pos_min = min([abs(pos - bound) for bound in read_bounds[2:]])
+                else:
+                    raise ValueError(
+                        'error getting end proximity - variant outside read bounds')
+            # update current min
+            if current_min is None: # `not current_min` would also encompass 0
+                current_min = pos_min
+            elif current_min > pos_min: # new min found
+                current_min = pos_min
+            elif current_min <= pos_min:
+                continue
+        self.min_end_proximity = current_min
 
         # get indels for calculation of indel attributes
         indels = []
@@ -662,7 +668,7 @@ class Pair():
             if tract[0] == self.converted_haplotype][0][1:])
 
 
-    def _get_mismatch_variant_ratio(self):
+    def _get_mismatch_variant_ratio(self, mvr_vcf_filepath):
         """
         Helper function to calculate mismatch_variant_ratio quality metric. Will
         use MD tags in reads to get the number of mismatches to the reference
@@ -676,12 +682,47 @@ class Pair():
         The MD tag also includes deletions, designated as '^ATG' if 'ATG' is a
         3 bp deletion from the reference in the read. These deletions are
         removed from consideration.
+
+        Can optionally take in the path to a VCF that has not yet been filtered
+        by `readcomb-vcfprep` - will apply default vcfprep filters _except_ for
+        the informative site (e.g. alleles differ between parents) filter so
+        that sites that differ from the reference but are invariant between
+        parents can still be used in mismatch_variant_ratio calc. This prevents
+        regions that are divergent from the reference but not from one another
+        from being penalized with high mismatch_variant_ratio values.
+
+        Parameters
+        ----------
+        mvr_vcf_filepath : str
+            optional path to VCF not yet filtered by vcfprep for MVR calc
         """
         if len(self.variants_1) + len(self.variants_2) == 0:
             return -1
         if len(self.variants_filt) == 0:
             return -1
 
+        if mvr_vcf_filepath: # provide VCF containing invariant sites
+            mvr_vcf_reader = VCF(mvr_vcf_filepath)
+            variants_1 = check_variants(
+                mvr_vcf_reader, self.rec_1.reference_name, self.rec_1.reference_start,
+                self.rec_1.reference_start + len(self.segment_1))
+            variants_2 = check_variants(
+                mvr_vcf_reader, self.rec_2.reference_name, self.rec_2.reference_start,
+                self.rec_2.reference_start + len(self.segment_2))
+            variants_all = [
+                [variant for variant in variants_1 + variants_2 if variant.POS == pos][0]
+                for pos in sorted(set(variant.POS for variant in variants_1 + variants_2))]
+            variants_filt_inv = [
+                variant for variant in variants_all
+                if variant.num_het == 0 and
+                len(variant.ALT) > 0 and
+                variant.is_snp and
+                all(variant.gt_quals >= 20)]
+            denominator = len(variants_filt_inv)
+        else:
+            denominator = len(self.variants_filt)
+
+
         if not self.overlap:
             rec_1_mismatches = len(
                 re.findall(
@@ -693,7 +734,7 @@ class Pair():
                     r'[ACGT]', re.sub(r'\^[ACGT]+', '', self.rec_2.get_tag('MD'))
                     )
                 )
-            return (rec_1_mismatches + rec_2_mismatches) / len(self.variants_filt)
+            return (rec_1_mismatches + rec_2_mismatches) / denominator
 
         elif self.overlap:
             # if rec2 is entirely 'contained' by rec1 - just do rec1
@@ -707,7 +748,7 @@ class Pair():
                         r'[ACGT]', re.sub(r'\^[ACGT]+', '', self.rec_1.get_tag('MD'))
                         )
                     )
-                return mismatches / len(self.variants_filt)
+                return mismatches / denominator
 
             # otherwise - handle overlap as normal
             m1 = self.rec_1.get_tag('MD')
@@ -746,7 +787,8 @@ class Pair():
                         r'[ACGT]', re.sub(r'\^[ACGT]+', '', region)
                         )
                     )
-            return mismatches / len(self.variants_filt)
+
+            return mismatches / denominator
 
     def _resolve_overlap(self):
         """
@@ -785,7 +827,12 @@ class Pair():
                 self.detection_1.remove(read_1_call)
                 self.detection_2.remove(read_2_call)
             elif read_1_call[0] == read_2_call[0]:
-                self.detection_2.remove(read_2_call) # only remove from right read
+                if int(read_2_call[-1]) > int(read_1_call[-1]): # qual is higher in right read
+                    # keep variant with higher qual for quality metrics
+                    self.detection_1.remove(read_1_call)
+                else:
+                    # qual is the same or higher in read 1 variant
+                    self.detection_2.remove(read_2_call) # only remove from right read
 
         # check if there is any overlap at all
         if not self.overlap:
